
// ------------------------------------ ANIMATION ------------------------------------

#import "constants.asm"
#import "macros.asm"

BasicUpstart2(main)

main:
	jsr setupRasterInterrupt

	lda #%11111111 // Enable multicolor for all sprites
	sta $d01c // Sprite multicolor

	lda #BLACK
	sta $d025 // Sprite extra color 1
	lda #WHITE
	sta $d026 // Sprite extra color 2


	// -------- Set sprite 0 values --------
	lda #RED
	sta $d027 // Sprite 0 color

	lda #100
	sta $d000 // X position

	lda #150
	sta $d001 // Y position

	lda #%00000001 // Enable sprite 0
	sta $d015 // Enable sprites



// ====================== GAME LOOP ======================
gameLoop:

	// ---- Animate sprite 0 ----
	ldx #0
	SetSpriteAnimation(0,10) // Animation type = 0    Animation speed = 10

	jmp gameLoop
// ========================================================



// ---------------------- Sprite animation subroutine -----------------------
spriteAnimation:
	ldx #0
	spriteAnimationLoop:

		updateSpriteAnimFrame:
			ldy Sprite_Anim_Type,x
			lda Sprite_Anim_LO,y
			sta ZP_ANIM_LO
			lda Sprite_Anim_HI,y
			sta ZP_ANIM_HI

			ldy Sprite_Anim_Frame_Counter,x
			lda (ZP_ANIM_LO),y
			sta Sprite_Anim_Frame,x
			cmp #255 // Animation end code
			bne notEndOfAnimation
				lda #0
				sta Sprite_Anim_Frame_Counter,x
				tay
				lda (ZP_ANIM_LO),y
				sta Sprite_Anim_Frame,x
			notEndOfAnimation:
				lda #SPRITE_POINTER_INDEX
				clc
				adc Sprite_Anim_Frame,x
				sta SPRITE_POINTER,x

		checkAnimTimer:
			lda Sprite_Anim_Timer,x
			cmp Sprite_Anim_Speed,x
			beq resetAnimTimer
				inc Sprite_Anim_Timer,x
				jmp doneCheckAnimTimer
			resetAnimTimer:
				lda #0
				sta Sprite_Anim_Timer,x

				inc Sprite_Anim_Frame_Counter,x
			doneCheckAnimTimer:

		inx
		cpx #8 // Max number of sprites
		beq doneSpriteAnimation
			jmp spriteAnimationLoop
	doneSpriteAnimation:
		rts
// --------------------------------------------------------------------------



// ---------------------- Raster interrupt routine -----------------------
irq:
	lda #%11111111
	sta INTERRUPT_STATUS
	
	jsr spriteAnimation // Run sprite animation subroutine
	
	jmp INTERRUPT_RETURN
// -----------------------------------------------------------------------


setupRasterInterrupt:
	sei

	lda #%01111111
	sta INTERRUPT_CONTROL_AND_STATUS1
	sta INTERRUPT_CONTROL_AND_STATUS2

	lda INTERRUPT_CONTROL_AND_STATUS1
	lda INTERRUPT_CONTROL_AND_STATUS2

	lda #240
	sta CURRENT_RASTER_LINE

	lda #%00011011
	sta SCREEN_CONTROL_1

	lda #<irq
	sta INTERRUPT_EXEC_ADDR1_SERVICE
	lda #>irq
	sta INTERRUPT_EXEC_ADDR2_SERVICE

	cli

	lda #%00000101
	sta INTERRUPT_CONTROL

	rts


enableRasterInterrupt:
	lda #%00000101
	sta INTERRUPT_CONTROL
	rts


disableRasterInterrupt:
	lda #%00000100
	sta INTERRUPT_CONTROL
	rts



*=SPRITE_PATTERN_ADDRESS "Sprite Patterns"
.import binary "spritePatterns.bin"


*=TABLES_ADDRESS "Tables"
#import "tables.asm"
